-- Upgrade script for multiple values in logbuch art
--
-- This is a *modified* copy of sql/upgrades/1.2.12/20_logbuch_art.sql
-- from db-admin 303:f78f1cd646aa. All references to the typ column have
-- been removed because on some struktur databases, this column doesn't
-- exist (see mpuls/issue6907, mpuls/issue7504).
--
-- Part of mpuls/issue7362

DROP VIEW ka_logbuch_tbl_view;

-- Create the nm table for the art values, fill it from ka_logbuch_tbl
-- and remove the art column from ka_logbuch_tbl.
--
-- "Keine Angabe", i.e. art == -1, is represented by an empty set of art
-- values after the migrations, so we omit those when populating
-- nm_logbuch_art_tbl
CREATE TABLE nm_logbuch_art_tbl (
    parent_id int4 NOT NULL REFERENCES ka_logbuch_tbl(id) ON DELETE CASCADE,
    choice_id int4 NOT NULL REFERENCES ka_logbuch_art_tbl(id),

    PRIMARY KEY (parent_id, choice_id)
);

INSERT INTO nm_logbuch_art_tbl (parent_id, choice_id)
     SELECT id, art FROM ka_logbuch_tbl
      WHERE art <> -1;

ALTER TABLE ka_logbuch_tbl DROP COLUMN art;


--
-- Adapt functions that create logbook entries
--
CREATE OR REPLACE FUNCTION master_ds_editor_changed() RETURNS trigger AS $master_ds_editor_changed$
DECLARE
    lid      integer;
    old_user RECORD;
    new_user RECORD;
BEGIN
    IF OLD.bearbeiter_id <> NEW.bearbeiter_id THEN

        SELECT INTO old_user * FROM ka_benutzer_tbl WHERE id = OLD.bearbeiter_id;
        SELECT INTO new_user * FROM ka_benutzer_tbl WHERE id = NEW.bearbeiter_id;

        SELECT INTO lid create_logbuch_ds(NEW.id, NULL);

        UPDATE ka_logbuch_tbl_view
            SET art = ARRAY[0],
            notiz = notiz_from_plaintext('Fallakte übertragen von ' 
                    || coalesce(old_user.nachname, '-/-') || ', '
                    || coalesce(old_user.vorname,  '-/-') || ' an '
                    || coalesce(new_user.nachname, '-/-') || ', '
                    || coalesce(new_user.vorname,  '-/-'))
        WHERE
            id = lid;
    END IF;
    RETURN NEW;
END;
$master_ds_editor_changed$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION master_ds_created_logbook() RETURNS trigger AS $master_ds_created_logbook$
DECLARE
    lid integer;
BEGIN
    SELECT INTO lid create_logbuch_ds(NEW.id, NULL);
    UPDATE ka_logbuch_tbl_view
        SET art = ARRAY[0],
            notiz = notiz_from_plaintext('Fallakte angelegt.')
    WHERE
        id = lid;
    RETURN NEW;
END;
$master_ds_created_logbook$ LANGUAGE plpgsql;


--
-- Recreate the view and rules
--

CREATE OR REPLACE VIEW ka_logbuch_tbl_view
    AS SELECT
        ka_logbuch_tbl.id         AS id,
        ka_logbuch_tbl.uuid_id    AS uuid_id,
        ka_logbuch_tbl.master_id  AS master_id,
        ka_logbuch_tbl.datum      AS datum,
        ka_logbuch_tbl.bearbeiter AS bearbeiter,
        ARRAY(SELECT choice_id
                FROM nm_logbuch_art_tbl nm
               WHERE nm.parent_id = ka_logbuch_tbl.id) AS art,
        ka_logbuch_tbl.kurz_notiz AS kurz_notiz,
        ka_logbuch_tbl.notiz      AS notiz,
        ka_logbuch_tbl.dauer      AS dauer
    FROM ka_logbuch_tbl
    INNER JOIN master_tbl ON
        ka_logbuch_tbl.master_id = master_tbl.id
    WHERE
        master_tbl.bearbeiter_id = getuserid()
        OR master_tbl.id IN (
            SELECT nm.master_id FROM nm_benutzer_master_tbl nm
            WHERE nm.benutzer_id IN (
                SELECT nm2.gruppe_id FROM nm_benutzer_gruppe_tbl nm2
                WHERE nm2.benutzer_id = getuserid())
        )
        OR pg_has_role('admin','MEMBER');


CREATE OR REPLACE RULE ka_logbuch_update_rule
    AS ON UPDATE TO ka_logbuch_tbl_view
    DO INSTEAD (
        UPDATE ka_logbuch_tbl SET
            datum      = new.datum,
            bearbeiter = new.bearbeiter,
            kurz_notiz = new.kurz_notiz,
            notiz      = new.notiz,
            dauer      = new.dauer
        WHERE ka_logbuch_tbl.id = new.id;
        DELETE FROM nm_logbuch_art_tbl WHERE parent_id = new.id;
        INSERT INTO nm_logbuch_art_tbl (parent_id, choice_id)
             SELECT new.id as parent_id, a.id as choice_id
               FROM ka_logbuch_art_tbl a
              WHERE a.id = ANY (new.art);
    );


CREATE OR REPLACE RULE ka_logbuch_delete_rule
    AS ON DELETE TO  ka_logbuch_tbl_view DO INSTEAD NOTHING;

CREATE OR REPLACE RULE ka_logbuch_insert_rule
    AS ON INSERT TO ka_logbuch_tbl_view DO INSTEAD NOTHING;

GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :cm_group;
GRANT UPDATE ON ka_logbuch_tbl_view TO GROUP :cm_group;
GRANT SELECT ON ka_logbuch_tbl_view TO GROUP :admin_group;


CREATE OR REPLACE VIEW nm_logbuch_art_tbl_view
    AS SELECT parent_id,
              choice_id
        FROM nm_logbuch_art_tbl nm
        JOIN ka_logbuch_tbl_view l ON l.id = nm.parent_id;

GRANT SELECT ON nm_logbuch_art_tbl_view TO GROUP :cm_group;
GRANT SELECT ON nm_logbuch_art_tbl_view TO GROUP :admin_group;


-- Delete obsolete choices
--
-- -1, i.e. "Keine Angabe" is not used anymore.
DELETE FROM ka_logbuch_art_tbl
      WHERE id = -1;
