-- Upgrade script for JMD-iQ related changes
--
-- mpuls/issue7420

--
-- From install-struktur/01master.sql
--

CREATE TABLE mikroprojekt_format_tbl (
    id INTEGER PRIMARY KEY NOT NULL,
    value VARCHAR(26) NOT NULL
);
CREATE TABLE mikroprojekt_handlungsfeld_tbl (
    id INTEGER PRIMARY KEY NOT NULL,
    value VARCHAR(70) NOT NULL
);

ALTER TABLE master_tbl
  ADD COLUMN jmd_iq INTEGER DEFAULT -1 REFERENCES entstand_aus_kopie_tbl (id);


CREATE TABLE rg_jmd_iq_tbl (
    id SERIAL PRIMARY KEY NOT NULL,
    master_id INTEGER NOT NULL,
    uuid_id CHAR(36),
    kopie_geprueft_p_iq INTEGER DEFAULT -1,
    name_mikroprojekt VARCHAR(60),
    kurzbeschreibung_mikroprojekt TEXT,
    datum_beginn_mikroprojekt DATE,
    datum_ende_mikroprojekt DATE,
    anzahl_iq_teilnehmende_direkt_maennlich INTEGER,
    anzahl_iq_teilnehmende_direkt_weiblich INTEGER,
    anzahl_iq_teilnehmende_direkt_divers INTEGER,
    anzahl_iq_teilnehmende_indirekt INTEGER,
    FOREIGN KEY (master_id) REFERENCES master_tbl (id) ON DELETE CASCADE,
    UNIQUE (uuid_id),
    FOREIGN KEY (kopie_geprueft_p_iq) REFERENCES entstand_aus_kopie_tbl (id)
);
CREATE TABLE nm_mikroprojekt_handlungsfeld_tbl (
    rg_jmd_iq_id INTEGER NOT NULL,
    choice_id int4 NOT NULL,
    FOREIGN KEY (rg_jmd_iq_id) REFERENCES rg_jmd_iq_tbl (id) ON DELETE CASCADE,
    FOREIGN KEY (choice_id) REFERENCES mikroprojekt_handlungsfeld_tbl (id),
    PRIMARY KEY (rg_jmd_iq_id, choice_id)
);
CREATE TABLE nm_mikroprojekt_format_tbl (
    rg_jmd_iq_id INTEGER NOT NULL,
    choice_id int4 NOT NULL,
    FOREIGN KEY (rg_jmd_iq_id) REFERENCES rg_jmd_iq_tbl (id) ON DELETE CASCADE,
    FOREIGN KEY (choice_id) REFERENCES mikroprojekt_format_tbl (id),
    PRIMARY KEY (rg_jmd_iq_id, choice_id)
);

INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (-2, 'Sonstiges');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (1, 'AG');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (2, 'Ausflug');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (3, 'Veranstaltung');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (4, 'Stadt- oder Quartiersfest');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (5, 'Veranstaltungsreihe');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (6, 'Workshop');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (7, 'Workshopreihe');
INSERT INTO mikroprojekt_format_tbl (id, value) VALUES (8, 'Onlineformat');

INSERT INTO mikroprojekt_handlungsfeld_tbl (id, value) VALUES (1, 'Bildung');
INSERT INTO mikroprojekt_handlungsfeld_tbl (id, value) VALUES (2, 'Sprache');
INSERT INTO mikroprojekt_handlungsfeld_tbl (id, value) VALUES (3, 'Arbeitsmarkt, Ausbildung und Armutsprävention');
INSERT INTO mikroprojekt_handlungsfeld_tbl (id, value) VALUES (4, 'Verbraucherschutz');
INSERT INTO mikroprojekt_handlungsfeld_tbl (id, value) VALUES (5, 'Gesundheit und Prävention');
INSERT INTO mikroprojekt_handlungsfeld_tbl (id, value) VALUES (6, 'Unterstützung partizipativer Projekte von jungen Menschen im Quartier');

GRANT SELECT ON mikroprojekt_format_tbl TO GROUP :cm_group;
GRANT SELECT ON mikroprojekt_format_tbl TO GROUP :admin_group;
GRANT SELECT ON mikroprojekt_handlungsfeld_tbl TO GROUP :cm_group;
GRANT SELECT ON mikroprojekt_handlungsfeld_tbl TO GROUP :admin_group;


--
-- From install-struktur/02rgroup.sql
--

-- Python is untrusted by default, we need to be postgres for this step
SET ROLE postgres;

-- DROP FUNCTION compute_case_structure(int4);
CREATE OR REPLACE FUNCTION compute_case_structure(case_id int4) RETURNS TEXT
AS $$

class Node(object):

    def __init__(self, name, id=None, children=[]):
        self.name = name
        self.id = id
        self.children = children

    def to_json(self, out):
        out.append('{"name":"%s","id":%d' % (self.name, self.id))
        if self.children:
            out.append(',"children":[')
            first = True
            for child in self.children:
                if first:
                    first = False
                else:
                    out.append(",")
                child.to_json(out)
            out.append("]")
        out.append("}")

    def recursive_build(self, mid, children, parent_name="master"):
        select = ("SELECT id FROM %s WHERE %s_id = %d"
                  % (self.name, parent_name, mid))
        ids = [r['id'] for r in plpy.execute(select)]
        for id in ids:
            sub_children = []
            node = Node(self.name, id, sub_children)
            name = self.name[:-4] # cut off trailing '_tbl'
            for child in self.children:
                child.recursive_build(id, sub_children, name)
            children.append(node)

TREE_STRUCTURE = Node('master_tbl', children = [
    Node('rg_netzwerk_tbl'),
    Node('rg_koop_1_tbl'),
    Node('rg_koop_2_tbl'),
    Node('rg_koop_3_tbl'),
    Node('rg_koop_4_tbl'),
    Node('rg_koop_5_tbl'),
    Node('rg_koop_bewertungen_tbl'),
    Node('rg_kjp_gruppenangebote_tbl'),
    Node('rg_nichtkjp_gruppenangebote_tbl'),
    Node('rg_jmd_iq_tbl')
])


children = []
tree = Node(TREE_STRUCTURE.name, case_id, children)
for child in TREE_STRUCTURE.children:
    child.recursive_build(case_id, children)
out = []
tree.to_json(out)
return "".join(out)

$$ LANGUAGE plpythonu EXTERNAL SECURITY DEFINER;
ALTER FUNCTION compute_case_structure(int4) OWNER TO :adm_ka_owner;

-- Rest as db-adm again
SET ROLE :db_adm;


CREATE OR REPLACE FUNCTION rg_jmd_iq_tbl_insert_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_jmd_iq_tbl_insert_trigger AFTER INSERT ON rg_jmd_iq_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_jmd_iq_tbl_insert_func();

CREATE OR REPLACE FUNCTION rg_jmd_iq_tbl_update_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid NEW.master_id;
    PERFORM set_lastaccess_time(mid);
    RETURN NEW;
END;
$$ LANGUAGE plpgsql EXTERNAL SECURITY DEFINER;

CREATE TRIGGER rg_jmd_iq_tbl_update_trigger AFTER UPDATE ON rg_jmd_iq_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_jmd_iq_tbl_update_func();

CREATE OR REPLACE FUNCTION rg_jmd_iq_tbl_delete_func() RETURNS TRIGGER AS $$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid OLD.master_id;
    UPDATE case_structure
        SET modified = FALSE, cache = compute_case_structure(master_id)
        WHERE master_id = mid;
    PERFORM set_lastaccess_time(mid);
    RETURN OLD;
END;
$$ LANGUAGE plpgsql;

CREATE TRIGGER rg_jmd_iq_tbl_delete_trigger AFTER DELETE ON rg_jmd_iq_tbl
    FOR EACH ROW EXECUTE PROCEDURE rg_jmd_iq_tbl_delete_func();


CREATE OR REPLACE FUNCTION create_rg_jmd_iq_tbl(pid integer, nuuid char(36))
RETURNS integer
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    rid integer;
    mid integer;
BEGIN
    mid := pid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    INSERT INTO rg_jmd_iq_tbl (master_id) VALUES (pid);

    SELECT INTO rid currval('rg_jmd_iq_tbl_id_seq');

    IF nuuid IS NOT NULL THEN
        UPDATE rg_jmd_iq_tbl
        SET    uuid_id = nuuid
        WHERE  id = rid;
    ELSE
        PERFORM set_uuid('rg_jmd_iq_tbl', rid);
        
    END IF;

    RETURN rid;
END;
$$;

CREATE OR REPLACE FUNCTION delete_rg_jmd_iq_tbl(rid integer)
RETURNS void
LANGUAGE 'plpgsql' VOLATILE EXTERNAL SECURITY DEFINER AS
$$
DECLARE
    mid integer;
BEGIN
    SELECT INTO mid master_id FROM rg_jmd_iq_tbl WHERE id = rid;

    IF NOT isEditorOrStandin(mid) THEN
        RAISE EXCEPTION 'DS ID not session_user ID';
    END IF;

    DELETE FROM rg_jmd_iq_tbl WHERE id = rid;
END;
$$;
