# -*- coding: utf-8 -*-
#
# Copyright 2018 Intevation GmbH, Germany, <info@intevation.de>
#
# This file is part of mpuls WASKA (CoMPUter-based case fiLeS -
# Web-Anwendungs-Server fuer Kompetenzagenturen).
#
# mpuls WASKA is free software: you can redistribute it and/or modify it under
# the terms of the GNU Affero General Public License as published by the
# Free Software Foundation, either version 3 of the License, or (at your
# option) any later version.
#
# mpuls WASKA is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
# FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Affero General Public
# License for more details.
#
# You should have received a copy of the GNU Affero General Public
# License along with mpuls WASKA. If not, see <http://www.gnu.org/licenses/>.
#
# mpuls WASKA has been developed on behalf of the
# Projekttraeger im Deutschen Zentrum fuer Luft- und Raumfahrt e.V. (PT-DLR)
# within the programme Kompetenzagenturen (Durchfuehrungsphase) funded by
# the Bundesministerium fuer Familie, Senioren, Frauen und Jugend and
# European Social Fund resources.
#
# Authors:
# Ludwig Reiter <ludwig.reiter@intevation.de>

from pylons.controllers.util import redirect, abort

from mpulsweb.controllers.appointment import (
    AppointmentController as MpulsAppointmentController
)
from mpulsweb.lib.base import render, c, session, request, restrict_to_POST
from mpulsweb.lib.helpers import url
from mpulsweb.lib.security import checkRole, hasRole
from mpulsweb.lib.translation import _
from mpulsweb.model.appointment import (
    GlobalAppointmentOverview,
    CaseAppointmentOverview,
    CaseReminderOverview
)

from jmdweb.controllers.mpuls import get_jmd_reminders


class AppointmentController(MpulsAppointmentController):

    @checkRole(('admin', 'cm', 'pt_dlr'))
    def overview(self):
        c.globalappointmentlist = GlobalAppointmentOverview(
            use_trigger=self._get_use_trigger_and_done(),
            use_done_check=self._get_use_trigger_and_done())
        c.appointmentlist = CaseAppointmentOverview(
            use_trigger=self._get_use_trigger_and_done(),
            use_done_check=self._get_use_trigger_and_done())
        c.followuplist = CaseReminderOverview(
            use_trigger=self._get_use_trigger_and_done(),
            use_done_check=self._get_use_trigger_and_done())
        if 'success_text' in session:
            c.success_text = session['success_text']
            del session['success_text']
            session.save()
        return render('/appointments/appointment_overview.mako')

    @checkRole(('admin', 'cm', 'pt_dlr'))
    def reminder_overview(self):
        use_trigger_and_done = self._get_use_trigger_and_done()
        c.remindlist = get_jmd_reminders(
            use_trigger=use_trigger_and_done,
            use_done_check=use_trigger_and_done,
            consider_seen_flag=use_trigger_and_done)
        c.show_set_done = True
        c.set_done_backlink = "reminder_overview"

        # Set success_text
        if 'success_text' in session:
            c.success_text = session['success_text']
            del session['success_text']
            session.save()

        return render('/appointments/reminder_overview.mako')

    def _get_use_trigger_and_done(self):
        use_trigger_and_done = True
        if hasRole('admin'):
            use_trigger_and_done = False
            c.show_set_done_title = True
        return use_trigger_and_done

    def _handle_backlink(self, backlink):
        if backlink == "reminder_overview":
            c.success_text = _(u"The appointment was successfully set done.")
            return self.reminder_overview()
        elif backlink == "appointment_overview":
            session['success_text'] = \
                _(u"The appointment was successfully set done.")
            session.save()
            redirect(url(controller="appointment", action="overview"))
        else:
            return super(AppointmentController, self)._handle_backlink(backlink)

    @checkRole('cm')
    @restrict_to_POST
    def age_reminder_seen(self, id):
        case = self._loadCase(id)
        instance_tree = case.get_formed_instance_tree()
        key = "wdv_altersgrenze_6_monate_gesehen:%d" % (case.id,)
        instance_tree.setData({key: "1"})
        instance_tree.commit()
        return self._handle_backlink(request.params.get("backlink"))
