/*
 * simple_countdown.js
 *
 * Frank Koormann <frank@intevation.de>
 *
 * A very simple countdown (e.g. to display the time until session timeout:
 * - Initialize with seconds to count down
 * - Each second the countdown updates the "simpleCountdown" element's
 *   inner HTML with the remaining time
 * - The time will be formatted as "mm min" (updated each minute) with a
 *   configurable threshold when changing to "mm:ss" (updated each second)
 * - A configurable Text will be displayed, when the countdown is over
 */

// Global configuration
var sc_remaining_time,
    sc_threshold = 180,
    sc_timeout_message = 'EXPIRED';

// Make the interval accessible
var sc_interval;

/* sc_formatTime(time, threshold)
 * Format time ("mm min" by default and "mm:ss" when under threshold
 *   time      : time to format in seconds
 *   threshold : threshold to switch format in seconds
 */
function sc_formatTime(time, threshold) {
    var min = parseInt(time / 60),
        sec = time - (min * 60)
    if (time >= threshold) {
        return min + " min";
    } else {
        return (min > 0 ? sc_pad(min, 2) : "00") + ":" + sc_pad(sec,2);
    }
}

/* pad(number, length
 * Format (int) number with leading "0"s, e.g.
 *   number     : integer number
 *   length     : length to be filled up with leading "0"
 */
function sc_pad(number, length) {
    var str = '' + number;
    while (str.length < length) {str = '0' + str;}
    return str;
}

/* sc_countdown()
 * Core function called by the interval (each second) to update the countdown
 */
function sc_countdown() {

    // Decrement the remaining time
    sc_remaining_time = sc_remaining_time - 1;

    // Update the displayed time
    if (sc_remaining_time > 0) {
        document.getElementById("simpleCountdown"
                                ).innerHTML = sc_formatTime(sc_remaining_time,
                                                            sc_threshold);
    } else {
    // clear the interval and display timeout message when time is over.
        clearInterval(sc_interval);
        document.getElementById("simpleCountdown"
                                ).innerHTML = sc_timeout_message;
    }
}

/* sc_init_simple_countdown(time, threshold, message)
 * Initialize the countdown. Call this function in your application:
 *   time      : time to count down (in seconds)
 *   threshold : threshold to switch format (in seconds)
 *   message   : Message to display (instead of countdown) when time is over
 */
function sc_init_simple_countdown(time, threshold, message) {

    sc_remaining_time = time;
    sc_threshold = threshold;
    sc_timeout_message = message;

    document.getElementById("simpleCountdown"
                                ).innerHTML = sc_formatTime(sc_remaining_time,
                                                            sc_threshold);

    // Set interval and update the count down every 1 second
    sc_interval = setInterval(sc_countdown,
                              1000);
}
