## -*- coding: utf-8 -*-
<%!

from pylons import app_globals as g
import mpulsweb.lib.helper.filters as F
import mpulsweb.lib.helpers as h
from mpulsweb.lib.translation import _
from formed.meta.document import containing_repeat_group

fields = g.case.get_overview_fields_list()

# Extend fields with default fields in overview
fields.extend([
    (_('Editor'), "bearbeiter_id", "table_name",
     ["admin"],
     lambda case: "%s, %s" % (F.NA(case.getEditor().last_name),
                              F.NA(case.getEditor().first_name[:1]))),

    (_('Updated'), "zugriff", "table_name", ["cm"],
     lambda case: h.format_date(case.state.getAccessTime())),
    ("ID", "id", "table_width_mid_s", ["cm", "admin"],
     lambda case: F.NA(case.id)),
  ])
%>

<%def name="build_list_headers()">
  <%
    # Normally CASE_OVERVIEW_SEARCHOPTIONS should contain at least the default
    # ordering fields when showing the case-overview (Set in
    # case_overview-controller). But as this template is included in other
    # use-cases too this is not always true. In theses cases be more error
    # tolerant and fallback to no (system default = id) sorting,
    # which should be ok in theses cases.
    s = session.get('CASE_OVERVIEW_SEARCHOPTIONS', {})
    sort_field = s.get('sort_field')
    sort_order = s.get('sort_order')
  %>
  % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
    <th class="table_header_h table_checkbox">
      <input type="checkbox" name="mpuls-toggle-all-case_id"
         id="mpuls-toggle-all-case_id" title="${_('Select/deselect all cases')}">
    </th>
  % endif
  % for title, urlpart, style, roles, get_value in fields:
    % if h.hasRole(roles):
      <th id="table-overview" class="${style} table_header_h">
        <div class="row">
          <div class="col">
        ${title}
        <%
          nc = g.formedTree.name2nc.get(urlpart)
          hide_sorting = nc and containing_repeat_group(nc)
        %>
          </div>
        </div>
        % if not hide_sorting:
        <div class="row d-print-none">
        <div class="col">
        <a title='${_("Sort in ascending order")}' href="${h.url(controller='case_overview', action='overview',
                             sort_field=urlpart, sort_order='asc')}">

            %if sort_field == urlpart and sort_order == 'asc':
              <svg class="svg-sort-active" aria-labelledby="title" role="img">
                <title id="title">${_("Sorted by %s in ascending order") % title}</title>
            %else:
              <svg class="svg-sort" aria-labelledby="title" role="img">
                <title id="title">${_("Sort in ascending order")}</title>
            %endif
            <use xlink:href="#sort_asc">
          </svg>
        </a>
        <a title='${_("Sort in descending order")}' href="${h.url(controller='case_overview', action='overview',
                             sort_field=urlpart, sort_order='desc')}">
            %if sort_field == urlpart and sort_order == 'desc':
              <svg class="svg-sort-active" aria-labelledby="title" role="img">
                <title id="title">${_("Sorted by %s in descending order") % title}</title>
            %else:
              <svg class="svg-sort" aria-labelledby="title" role="img">
                <title id="title">${_("Sort in descending order")}</title>
            %endif
            <use xlink:href="#sort_desc">
          </svg>
        </a>
        </div>
        </div>
        % endif
      </th>
    % endif
  % endfor
  <th id="table-overview" class="table_status table_header_h">
    ${_('Status')}
  </th>
  <th id="table-overview" class="table_action table_header_h">
    ${_('Actions')}
  </th>
</%def>

<%def name="build_list_row(case)">
  % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
    <td>
        <input type="checkbox" name="case_id" value="${case.id}">
        <a class="caseoverview_anchor" id="${case.id}"></a>
    </td>
  % endif
  % for title, urlpart, style, roles, get_value in fields:
    % if h.hasRole(roles):
      % if case.getState().getState() != 5 and case.getState().getState() != 3 and case.getState().getState() != 4:
        <%
           nc = g.formedTree.name2nc.get(urlpart)
           if not nc:
               link_url = h.url(controller='case', action='select', id=case.id,
                                confirmed=0)
           else:
               link_url = h.url(controller='case', action='select_field',
                                id=case.id, confirmed=urlpart)
        %>
        <td style="transform: rotate(0);"><a href="${link_url}" class="stretched-link">${get_value(case) | F.NA}</a></td>
      % else:
        <td>${get_value(case) | F.NA}</td>
      % endif
    % endif
  % endfor
  <td class="phase_icon">
    ${get_phase_icon(case)}
    % if g.mpuls_config.is_enabled('case-module', 'tags'):
        ${get_validity_icon(case)}
    % endif
    ${get_state_icon(case)}
  </td>
  <td class="icon-td">
    ${get_actions(case)}
  </td>
</%def>

<%def name="get_validity_icon(case)">
  % for validity in g.mpuls_config.get('validities', 'enabled'):
    % if validity['id'] == case.getValidity():
      ##<a href="${h.url(controller='tags', action='overview', id=case.id)}">
        <img src="${h.url('/images/icons/%s' % validity['icon'])}" alt="${validity['name']}" title="${validity['name']}">
      ##</a>
    % endif
  % endfor
</%def>

<%def name="get_phase_icon(case)">
  <% phase = c.statemachine.get_current_state(case) %>
  <% phase_id = phase.get_id() %>
  <% running = not phase_id%2 and 'running' or 'finished' %>
  <% description = h.literal(phase.get_display_name()) %>
  <% icon = phase.get_icon_name() %>
  <span title="${_('Status')}: ${description}">
  <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
    <title id="title">${_('Status')}: ${description}</title>
    <use xlink:href="#${icon}_${running}" />
  </svg>
  </span>
</%def>

<%def name="render_phase_legend()">
<div class="info float-right">
  <strong>${_('Legend')}: </strong>
  % for short_name, long_name in c.statemachine.get_state_legend():
        ${short_name}: ${long_name},
  % endfor

  | <span title="${_('Status')}: ${_('Running')}">
    <svg class="svg-icon-disabled-small" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Running')}</title>
      <use xlink:href="#phase-running" />
    </svg>
    </span>
    : ${_('Running')},
     <span title="${_('Status')}: ${_('Finished')}">
     <svg class="svg-icon-disabled-small" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Finished')}</title>
      <use xlink:href="#phase-stop" />
    </svg>
    </span>
    : ${_('Finished')}
</div>
<br>
</%def>

<%def name="get_state_icon(case)">
  % if h.hasRole(['cm']) and g.mpuls_config.is_enabled('case-module', 'organisation'):
    % if case.user_is_editor(session.get('USER_AUTHORIZED').id):
    <span title="${_('Status')}: ${_('Editor')}">
      <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
        <title id="title">${_('Status')}: ${_('Editor')}</title>
        <use xlink:href="#edit_editor" />
      </svg>
    </span>
    % else:
    <span title="${_('Status')}: ${_('Standin')}">
      <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
        <title id="title">${_('Status')}: ${_('Standin')}</title>
        <use xlink:href="#rep" />
      </svg>
    </span>
    % endif
  % endif
  % if case.getState().getState() in (1,2):
  <span title="${_('Status')}: ${_('Editable')}">
    <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Editable')}</title>
      <use xlink:href="#edit" />
    </svg>
  </span>
  % elif case.getState().getState() == 3:
  <span title="${_('Status')}: ${_('Mark delete')}">
    <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Mark delete')}</title>
      <use xlink:href="#delete" />
    </svg>
  </span>
  % elif case.getState().getState() == 4:
  <span title="${_('Status')}: ${_('Mark anonym')}">
    <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Mark anonym')}</title>
      <use xlink:href="#anonymise" />
    </svg>
  </span>
  % elif case.getState().getState() == 5:
  <span title="${_('Status')}: ${_('Mark anonymise')}">
    <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Mark anonymise')}</title>
      <use xlink:href="#anonymise" />
    </svg>
  </span>
  % endif
  % if case.state.get_changed_editor():
  <span title="${_('Status')}: ${_('Changed editor')}">
    <svg class="svg-icon-disabled" aria-labelledby="title" role="img">
      <title id="title">${_('Status')}: ${_('Changed editor')}</title>
      <use xlink:href="#edit_changed_editor" />
    </svg>
  </span>
  % endif
</%def>

<%def name="get_actions(case)">
  % if h.hasRole(['admin']):
    % if case.getState().getState() in (3, 5) and g.mpuls_config.is_enabled('case-module', 'show-delete-icon'):
      <a title="${_('Status')}: ${_('Delete case')}" class="btn btn-outline-secondary btn-sm icon"
         href="${h.url(controller='case', action='delete', id=case.id,
                           confirmed=0)}">
        <svg class="svg-icon svg-warning" aria-labelledby="title" role="img">
          <title id="title">${_('Status')}: ${_('Delete case')}</title>
          <use xlink:href="#delete" />
        </svg>
       </a>
    % endif
    % if case.getState().getState() == 4:
      % if g.mpuls_config.is_enabled('case-module', 'anonymize'):
        <a title="${_('Status')}: ${_('Anonymize case document')}" href="${h.url(controller='case', action='anonymize', id=case.id,
                             confirmed=0)}">
          <svg class="svg-icon svg-warning" aria-labelledby="title" role="img">
            <title id="title">${_('Status')}: ${_('Anonymize case document')}</title>
            <use xlink:href="#anonymise" />
          </svg>
        </a>
      % endif
    % endif
    % if case.getState().getState() in (3, 4):
      <a title="${_('Restore case document')}" href="${h.url(controller='case', action='restore', id=case.id,
                           confirmed=0)}">
          <svg class="svg-icon" aria-labelledby="title" role="img">
            <title id="title">${_('Restore case document')}</title>
            <use xlink:href="#refresh" />
          </svg>
       </a>
    % endif
  % elif h.hasRole(['cm']) and case.getState().getState() in (1, 2):
    % if g.mpuls_config.is_enabled('case-module', 'anonymize'):
      <% sm = g.case_factory.get_state_machine() %>
      % if case.getState().getPhase() in sm.get_end_state_ids():
        <a title="${_('Anonymize case document')}"
               href="${h.url(controller='case',
               action='markForAnonymizeFromOverview', id=case.id,
               confirmed=0)}">
          <svg class="svg-icon svg-warning" aria-labelledby="title" role="img">
            <title id="title">${_('Anonymize case document')}</title>
            <use xlink:href="#anonymise" />
          </svg>
        </a>
      % endif
    % endif
    % if g.mpuls_config.is_enabled('case-module', 'show-delete-icon'):
      <a class="btn btn-outline-secondary btn-sm icon" title="${_('Delete')}"
         href="${h.url(controller='case', action='markForDelete', id=case.id,
                           confirmed=0)}">
        <svg class="svg-icon svg-warning" aria-labelledby="title" role="img">
          <title id="title">${_('Delete')}</title>
          <use xlink:href="#delete" />
        </svg>
      </a>
    % endif
  % endif
</%def>

<%def name="showAppointmentURL(case)">
"${h.url(controller='case', action='select', id=case.id, confirmed=0)}"
</%def>

<div class="table-responsive">
  <table id="table_caseoverview" class="table table-sm table-hover
         custom-table-striped">
    <thead>
      <tr>
       <th scope="col">${build_list_headers()}</th>
      </tr>
    </thead>
    <tbody>
    % for num, case in enumerate(c.cases.getDatasets()):
      % if h.hasRole(['admin']) and case.getState().getState() != 5:
        <tr class='clickable-row' data-href=${self.showAppointmentURL(case)}>
      % else:
        <tr>
      % endif
          <td>${build_list_row(case)}</td>
        </tr>
    % endfor
    </tbody>
    <tfoot>
    % if len(c.cases.getDatasets()) <= 0:
    <tr>
      <td colspan="${len(fields)}">
        ${_('No data found.')}
      </td>
      <td></td>
    </tr>
    % else:
      % if g.mpuls_config.is_enabled('case-module', 'casebundle'):
      <tr>
        <td></td>
        <td class="table_checkbox">
          <input type="checkbox" name="mpuls-toggle-all-case_id"
                 id="mpuls-toggle-all-case_id"
                 title="${_('Select/deselect all cases')}">
        </td>
        <td colspan="${len(fields)+1}">
          ${_('Select/deselect all cases')}
          ${render_phase_legend()}
        </td>
      </tr>
      % endif
    % endif
    </tfoot>
  </table>
</div>
