#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Generate an XLSX file from a po-file.
Usage:
gettext-to-xlsx.py  PO_FILE  XLSX_FILE
"""

import sys
import optparse

import openpyxl
from babel.messages.pofile import read_po


def list_po_file(filename):
    catalog = read_po(open(filename))
    for msg in catalog:
        if msg.id == "":
            continue
        value = msg.string
        if msg.fuzzy:
            if msg.pluralizable:
                value = ("", "")
            else:
                value = ""

        comments = list(msg.auto_comments)

        if "\n" in msg.id:
            comments.insert(0, "mehrzeilig")

        comments = ";".join(comments)

        if msg.pluralizable:
            yield (msg.id[0], value[0], comments)
            yield (msg.id[1], value[1], "")
        else:
            yield (msg.id, msg.string, comments)

def sheet_write(ws, row, col, val):
    """Wrapper around openpyxl ws.cell()"""
    ws.cell(row=row+1, column=col+1, value=val)

def make_workbook(translations):
    workbook = openpyxl.Workbook()
    sheet = workbook.active
    sheet.title = u"Übersetzungen"
    sheet_write(sheet, 0, 0, u"Schlüssel")
    sheet_write(sheet, 0, 1, u"Übersetzung")
    sheet_write(sheet, 0, 2, u"Kommentare")

    for row_num, (key, value, comments) in enumerate(translations):
        if isinstance(key, tuple):
            key = key[0]
        sheet_write(sheet, row_num + 1, 0, key)
        sheet_write(sheet, row_num + 1, 1, value)
        sheet_write(sheet, row_num + 1, 2, comments)

    return workbook

def write_workbook(workbook, filename):
    workbook.save(open(filename, "w"))


def po_to_xls(filename, xls_filename):
    write_workbook(make_workbook(list_po_file(filename)), xls_filename)


def main():
    usage = "Usage: %prog [options] PO-FILE XLSX-FILE"
    # we don't really need to parse options, but using OptionParser
    # gives us standard behaviour, like a --help option, for free.
    parser = optparse.OptionParser(usage=usage)
    options, args = parser.parse_args()

    if len(args) != 2:
        print >>sys.stderr, ("The po-file and the xlsx-file must be given"
                             " as command line arguments")

    po_to_xls(args[0], args[1])


main()
